import { Injectable } from '@angular/core';
import { NatsService } from 'app/core/nats/services/nats.service';
import * as nats from 'nats.ws';

import { Error } from 'app/modules/proto/protobuf/rpc_pb';

import {
  ListMemberGameSettingsRequest,
  ListMemberGameSettingsResponse,
  CreateMemberGameSettingRequest,
  CreateMemberGameSettingResponse,
  UpdateMemberGameSettingRequest,
  UpdateMemberGameSettingResponse,
  DeleteMemberGameSettingRequest,
  DeleteMemberGameSettingResponse,
  GetMemberGameSettingResponse,
  GetMemberGameSettingRequest,
} from 'app/modules/proto/c2se/member_game_setting_pb';
import {
  SUBJECT_LIST_MEMBER_GAME_SETTINGS,
  SUBJECT_CREATE_MEMBER_GAME_SETTING,
  SUBJECT_UPDATE_MEMBER_GAME_SETTING,
  SUBJECT_DELETE_MEMBER_GAME_SETTING,
  SUBJECT_GET_MEMBER_GAME_SETTING,
} from 'app/modules/proto/c2se/backend/member_game_setting_pb';

@Injectable({
  providedIn: 'root',
})
export class MemberGameSettingService {
  /**
   * Constructor
   */
  constructor(private __natsService: NatsService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Accessors
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  createMemberGameSetting(): Promise<CreateMemberGameSettingResponse.Result> {
    return new Promise<CreateMemberGameSettingResponse.Result>(
      (resolve, reject) => {
        let req = new CreateMemberGameSettingRequest();

        this.__natsService
          .request<CreateMemberGameSettingResponse.Result>(
            SUBJECT_CREATE_MEMBER_GAME_SETTING,
            req.serializeBinary(),
            CreateMemberGameSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result, result.getMemberGameSetting());
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }

  listMemberGameSettings(): Promise<ListMemberGameSettingsResponse.Result> {
    return new Promise<ListMemberGameSettingsResponse.Result>(
      (resolve, reject) => {
        let req = new ListMemberGameSettingsRequest();

        this.__natsService
          .request<ListMemberGameSettingsResponse.Result>(
            SUBJECT_LIST_MEMBER_GAME_SETTINGS,
            req.serializeBinary(),
            ListMemberGameSettingsResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result, result.getMemberGameSettingsList());
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            reject(e);
          });
      }
    );
  }

  getMemberGameSetting(
    id: string
  ): Promise<GetMemberGameSettingResponse.Result> {
    return new Promise<GetMemberGameSettingResponse.Result>(
      (resolve, reject) => {
        let req = new GetMemberGameSettingRequest();
        req.setId(id);

        this.__natsService
          .request<GetMemberGameSettingResponse.Result>(
            SUBJECT_GET_MEMBER_GAME_SETTING,
            req.serializeBinary(),
            GetMemberGameSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result, result.getMemberGameSetting());
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }

  updateMemberGameSetting(
    req: UpdateMemberGameSettingRequest
  ): Promise<UpdateMemberGameSettingResponse.Result> {
    return new Promise<UpdateMemberGameSettingResponse.Result>(
      (resolve, reject) => {
        this.__natsService
          .request<UpdateMemberGameSettingResponse.Result>(
            SUBJECT_UPDATE_MEMBER_GAME_SETTING,
            req.serializeBinary(),
            UpdateMemberGameSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result, result.getMemberGameSetting());
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }

  deleteMemberGameSetting(): Promise<DeleteMemberGameSettingResponse.Result> {
    return new Promise<DeleteMemberGameSettingResponse.Result>(
      (resolve, reject) => {
        let req = new DeleteMemberGameSettingRequest();

        this.__natsService
          .request<DeleteMemberGameSettingResponse.Result>(
            SUBJECT_DELETE_MEMBER_GAME_SETTING,
            req.serializeBinary(),
            DeleteMemberGameSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result);
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }
}
